/*
 * FileFunctions.cpp - File functions for the MIDI Synthesizer Project
 *
 * Written by: Craig A. Lindley
 * Version: 1.0
 * Last Update: 09/10/2014
 *
 * Copyright (c) 2014 Craig A. Lindley
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "FileFunctions.h"
#include "SdFat.h"

extern SdFat sd;
extern SdFile file;

#define MAX_FILES       30
#define FILENAME_LENGTH 13

int numberOfFiles;

// Define FILENAME type
typedef char FILENAME[FILENAME_LENGTH];

// Declare array of filenames
FILENAME filenames[MAX_FILES];

// Enumerate MIDI files in root directory of SD card
void enumerateMIDIFiles(boolean displayFilenames) {

    char fn[13];
    boolean isMIDIFile;
    
    numberOfFiles = 0;
    
    // Clear filenames array
    memset(filenames, 0, sizeof(filenames));

    file.close();
    
    sd.vwd()->rewind();

    while (file.openNext(sd.vwd(), O_READ)) {
        // Read a filename
        file.getFilename(fn);

        // Is this a MIDI file having .mid extension ?
        isMIDIFile = strstr(fn, ".MID") != NULL;
        
        // If filename is MIDI file and not deleted, grab its name and store it
        if ((fn[0] != '_') && (fn[0] != '~') && isMIDIFile && (numberOfFiles < MAX_FILES)) {

            // Copy the MIDI filename to filenames array
            strcpy(filenames[numberOfFiles++], fn);
            
            if (displayFilenames) {
                Serial.println(fn);
                delay(20);
            }
        }
        file.close();
    }
}

// Get the filename of the MIDI file with specified index
void getMIDIFilename(int index, char *buffer) {

    if ((index < 0) || (index >= MAX_FILES) || (index >= numberOfFiles)) {
        index = 0;
    }
    strcpy(buffer, (char *) &filenames[index]);
}


